/*
 * flashFS.cpp
 *
 *  Created on: Apr 29, 2014
 *      Author: Wesley
 */



#include "flashFS.h"
#include <string.h>


#define MAX_FS_BLOCKS ((FLASH_SIZE - PREALLOCATED_SPACE) / BLOCKSIZE)
#define FIRST_FS_ADDR ( FLASH_SIZE - (NUM_FS_BLOCKS * BLOCKSIZE) )
long NUM_FS_BLOCKS = (FS_SIZE / BLOCKSIZE);


/*
 * OnChipFlash_GetFlashName
 * Returns the expected name of the flash chip in pName.
 */
void OnChipFlash_GetFlashName(char *pName )
{
	strcpy( pName, FLASH_NAME );
	return;
}


#if defined NANO54415

#include <NanoStdFileSupport.h>

#define BLOCKS_PER_DESC (DESCSIZE/BLOCKSIZE)

/****************************************************************************
 *
 * GetBufferBlockAddr
 *
 * Get a logical block physical relative address in flash
 * relsector - relative sector in the block (<sectorperblock)
 *
 * INPUTS
 *
 * block - block number
 * relsector - rel
 *
 * RETURNS
 *
 * relative physical address of block
 *
 ***************************************************************************/
static long GetBufferBlockAddr(long block, long relsector)
{
   long rc;
   if (block == 1) {
       block = BLOCKS_PER_DESC;
   }
   rc = block * BLOCKSIZE + relsector * SECTORSIZE;
   if (rc < 0 || rc > FS_SIZE)
   {
      rc = 0;
   }
   return (rc);
}



static long GetBlockAddr(long block, long relsector)
{
   return GetBufferBlockAddr(block, relsector) + FIRST_ADDR;
}




static BYTE StdFlashBuffer[FS_SIZE];

#ifdef EXTRA_SLOW_VERIFY
static BYTE CpyFlashBuffer[FS_SIZE];

void RawVerify(const char * name, unsigned long pos, int len)
{
	int i;
	NanoStdFlashInit(CpyFlashBuffer,FS_SIZE,FIRST_ADDR);
	for(i=0; i< FS_SIZE; i++)
	{
	if(StdFlashBuffer[i]!=CpyFlashBuffer[i])
		{
		 iprintf("Verify error at %d (%08x) Buf=%02X Flash = %02X\r\n",i,i,StdFlashBuffer[i],CpyFlashBuffer[i]);
		 iprintf("Error at %s for %08X %d\r\n",name,pos,len);
		 return;
		}
	}
	iprintf("%s ok at %08X for %d\r\n",name,pos,len);

}
#else
#define RawVerify(x,y,z) (void)0
#endif


/****************************************************************************
 *
 * EraseFlash
 *
 * INPUTS
 *
 * block - which block needs to be erased
 *
 * RETURNS
 * 0 - always
 *
 ***************************************************************************/
int NanoEraseFlash(long block)
{
   int i;
   long offset = GetBufferBlockAddr(block, 0);
   long eraseSize = ((block == 0) || (block == 1)) ? BLOCKSIZE*BLOCKS_PER_DESC : BLOCKSIZE;

   if ((block == 0) || (block == 1)) {
       long rawOffset = GetBlockAddr(block, 0);
       for (i = 0; i < BLOCKS_PER_DESC; i++) {
           NanoRawEraseFlash( rawOffset, BLOCKSIZE );
           rawOffset += BLOCKSIZE;
       }
   }
   else {
       long rawOffset = GetBlockAddr(block, 0);
       NanoRawEraseFlash( rawOffset, BLOCKSIZE );
   }


   for (i = 0; i < eraseSize; i++)
   {
      StdFlashBuffer[i + offset] = 0xFF;
   }
   RawVerify("Erase",offset,BLOCKSIZE);
   return 0;
}



/****************************************************************************
 *
 * WriteFlash
 *
 * Writing (programming) Flash device
 *
 * INPUTS
 *
 * data - where data is
 * block - which block is programmed
 * relsector - relative sector in the block (<sectorperblock)
 * size - length of data
 * relpos - relative position of data in block
 *
 * RETURNS
 *
 * 0 - if ok
 * 1 - if there was any error
 *
 ***************************************************************************/
int NanoWriteFlash(void *data, long block, long relsector, long size, long relpos)
{
   int i;
   long offset = (GetBufferBlockAddr(block, relsector) + relpos);
   unsigned char * pbData = (unsigned char *) data;
   long bo = GetBlockAddr(block, relsector);

   long start_p = (bo + relpos);
   long end_p = start_p + size;
   long tempStart_p, tempEnd_p;

   // Copy the flash write into the ram mirror
   for (i = 0; i < size; i++)
      StdFlashBuffer[i + offset] = pbData[i];

   // Adjust the programming area to cover full sectors in spi flash
   start_p = start_p & 0xFFFFFF00;  // round down to nearest 256 bytes
   end_p = end_p | 0xff;  // round up to nearest 256 bytes

   if (end_p & 1)    // make normal writes even lengths
      end_p++;

   if (end_p >= FLASH_SIZE)  // Check for end of physical flash
      end_p = (FLASH_SIZE - 1);

   // Do the actual write to spi flash
   size = end_p - start_p;
   tempStart_p = start_p;
   do {
      tempEnd_p = (size > BLOCKSIZE) ? (tempStart_p + BLOCKSIZE) : (tempStart_p + size);
      NanoRawWriteFlash(StdFlashBuffer + (offset & 0xFFFFFF00), tempStart_p, tempEnd_p - tempStart_p);
      offset += tempEnd_p - tempStart_p;
      size -= tempEnd_p - tempStart_p;
      tempStart_p = tempEnd_p;
   } while (tempEnd_p != end_p);
   RawVerify("Write",offset,BLOCKSIZE);

   return 0;
}





/****************************************************************************
 *
 * VerifyFlash
 *
 * Compares data with flash containes
 *
 * INPUTS
 *
 * data - where data is
 * block - which block is programmed
 * relsector - relative sector in the block (<sectorperblock)
 * size - length of data
 * relpos - relative position of data in block
 *
 * RETURNS
 *
 * 0 - if ok
 * 1 - if there was any error
 *
 ***************************************************************************/
int NanoVerifyFlash( void *data, long block, long relsector, long size, long relpos )
{
   unsigned short * d = ( unsigned short * )( StdFlashBuffer + GetBufferBlockAddr( block, relsector ) + relpos );
   unsigned short * s = ( unsigned short * ) ( data );
   long a;

   size++;

   size >>= 1;
   size <<= 1; //word align

   for ( a = 0; a < size; a += 2 )  // verify
   {
      if ( *d++ != *s++ )
      {
         return 1; // failed
      }
   }
   return 0; //ok
}



/****************************************************************************
 *
 * ReadFlash
 *
 * read data from flash
 *
 * INPUTS
 *
 * data - where to store data
 * block - block number which block to be read
 * blockrel - relative start address in the block
 * datalen - length of data in bytes
 *
 * RETURNS
 * 0 - if sucessfully read
 *
 ***************************************************************************/
int NanoReadFlash( void *data, long block, long blockrel, long datalen )
{
   long src = GetBufferBlockAddr( block, 0 ) + blockrel;
   fsm_memcpy( data, StdFlashBuffer+src, datalen );
   return 0;
}

/****************************************************************************
 *
 * fs_phy_OnChipFlash
 *
 * Identify a flash and fills FS_FLASH structure with data
 *
 * INPUTS
 *
 * flash - structure which is filled with data of flash properties
 *
 * RETURNS
 *
 * 0 - if successfully
 * 1 - flash not valid
 *
 ***************************************************************************/

int my_fs_phy_OnChipFlash( FS_FLASH *flash )
{
   flash->ReadFlash =  NanoReadFlash;       //read content
   flash->EraseFlash = NanoEraseFlash;     //erase a block
   flash->WriteFlash = NanoWriteFlash;     //write content
   flash->VerifyFlash =NanoVerifyFlash;   //verify content


   flash->maxblock = ( FS_SIZE / BLOCKSIZE ) - BLOCKSTART;
   flash->blocksize = BLOCKSIZE;
   flash->sectorsize = SECTORSIZE;
   flash->sectorperblock = SECTORPERBLOCK;
   flash->blockstart = BLOCKSTART;
   flash->descsize = DESCSIZE;
   flash->descblockstart = DESCBLOCKSTART;
   flash->descblockend = DESCBLOCKEND;
   flash->cacheddescsize = DESCCACHE;        // write cache size in descriptor

   NanoStdFlashInit(StdFlashBuffer,FS_SIZE,FIRST_ADDR);
   RawVerify("Init",0,0);

   return 0;
}



#else

/****************************************************************************
 *
 * GetBlockAddr
 *
 * Get a logical block physical relative address in flash
 * relsector - relative sector in the block (<sectorperblock)
 *
 * INPUTS
 *
 * block - block number
 * relsector - rel
 *
 * RETURNS
 *
 * relative physical address of block
 *
 ***************************************************************************/
static long GetBlockAddr( long block, long relsector )
{
//   long rc = FIRST_ADDR + block *BLOCKSIZE + relsector *SECTORSIZE;
//   if ( rc<FIRST_ADDR || rc>FLASH_SIZE )
//   {
//      rc = FIRST_ADDR;
//   }
//   return( rc );
   long rc = FIRST_FS_ADDR + block *BLOCKSIZE + relsector *SECTORSIZE;
   if ( rc<FIRST_FS_ADDR || rc>FLASH_SIZE )
   {
      rc = FIRST_FS_ADDR;
   }
   return( rc );
}

/****************************************************************************
 *
 * EraseFlash
 *
 * INPUTS
 *
 * block - which block needs to be erased
 *
 * RETURNS
 * 0 - always
 *
 ***************************************************************************/
int EraseFlash( long block )
{
   FlashErase( ( void * ) ( FS_FLASHBASE + GetBlockAddr( block, 0 ) ), BLOCKSIZE );
   return 0;
}



/****************************************************************************
 *
 * WriteFlash
 *
 * Writing (programming) Flash device
 *
 * INPUTS
 *
 * data - where data is
 * block - which block is programmed
 * relsector - relative sector in the block (<sectorperblock)
 * size - length of data
 * relpos - relative position of data in block
 *
 * RETURNS
 *
 * 0 - if ok
 * 1 - if there was any error
 *
 ***************************************************************************/
int WriteFlash( void *data, long block, long relsector, long size, long relpos )
{
   FlashProgram( ( void * ) ( FS_FLASHBASE + GetBlockAddr( block, relsector ) + relpos ),
                 data,
                 size );
   return 0;
}


/****************************************************************************
 *
 * VerifyFlash
 *
 * Compares data with flash containes
 *
 * INPUTS
 *
 * data - where data is
 * block - which block is programmed
 * relsector - relative sector in the block (<sectorperblock)
 * size - length of data
 * relpos - relative position of data in block
 *
 * RETURNS
 *
 * 0 - if ok
 * 1 - if there was any error
 *
 ***************************************************************************/
int VerifyFlash( void *data, long block, long relsector, long size, long relpos )
{
   unsigned short * d = ( unsigned short * )
                        ( FS_FLASHBASE + GetBlockAddr( block, relsector ) + relpos );
   unsigned short * s = ( unsigned short * ) ( data );
   long a;

   size++;

   size >>= 1;
   size <<= 1; //word align

   for ( a = 0; a < size; a += 2 )  // verify
   {
      if ( *d++ != *s++ )
      {
         return 1; // failed
      }
   }
   return 0; //ok
}


/****************************************************************************
 *
 * ReadFlash
 *
 * read data from flash
 *
 * INPUTS
 *
 * data - where to store data
 * block - block number which block to be read
 * blockrel - relative start address in the block
 * datalen - length of data in bytes
 *
 * RETURNS
 * 0 - if sucessfully read
 *
 ***************************************************************************/
int ReadFlash( void *data, long block, long blockrel, long datalen )
{
   long src = GetBlockAddr( block, 0 ) + blockrel;
   fsm_memcpy( data, ( char * ) ( FS_FLASHBASE + src ), datalen );
   return 0;
}


///****************************************************************************
// *
// * fs_phy_OnChipFlash
// *
// * Identify a flash and fills FS_FLASH structure with data
// *
// * INPUTS
// *
// * flash - structure which is filled with data of flash properties
// *
// * RETURNS
// *
// * 0 - if successfully
// * 1 - flash not valid
// *
// ***************************************************************************/
//int fs_phy_OnChipFlash( FS_FLASH *flash )
//{
//   flash->ReadFlash = ReadFlash;       //read content
//   flash->EraseFlash = EraseFlash;     //erase a block
//   flash->WriteFlash = WriteFlash;     //write content
//   flash->VerifyFlash = VerifyFlash;   //verify content
//
//
//   flash->maxblock = ( FS_SIZE / BLOCKSIZE ) - BLOCKSTART;
//   flash->blocksize = BLOCKSIZE;
//   flash->sectorsize = SECTORSIZE;
//   flash->sectorperblock = SECTORPERBLOCK;
//   flash->blockstart = BLOCKSTART;
//   flash->descsize = DESCSIZE;
//   flash->descblockstart = DESCBLOCKSTART;
//   flash->descblockend = DESCBLOCKEND;
//   flash->cacheddescsize = DESCCACHE;        // write cache size in descriptor
//
//   return 0;
//}

/****************************************************************************
 *
 * fs_phy_OnChipFlash
 *
 * Identify a flash and fills FS_FLASH structure with data
 *
 * INPUTS
 *
 * flash - structure which is filled with data of flash properties
 *
 * RETURNS
 *
 * 0 - if successfully
 * 1 - flash not valid
 *
 ***************************************************************************/
int my_fs_phy_OnChipFlash( FS_FLASH *flash )
{
   flash->ReadFlash = ReadFlash;       //read content
   flash->EraseFlash = EraseFlash;     //erase a block
   flash->WriteFlash = WriteFlash;     //write content
   flash->VerifyFlash = VerifyFlash;   //verify content

//#define FS_SIZE    (           512 * 1024 )512kb	(64kb blocks)
   flash->maxblock = ( NUM_FS_BLOCKS ) - BLOCKSTART;
   flash->blocksize = BLOCKSIZE;
   flash->sectorsize = SECTORSIZE;
   flash->sectorperblock = SECTORPERBLOCK;
   flash->blockstart = BLOCKSTART;
   flash->descsize = DESCSIZE;
   flash->descblockstart = DESCBLOCKSTART;
   flash->descblockend = DESCBLOCKEND;
   flash->cacheddescsize = DESCCACHE;        // write cache size in descriptor

   return 0;
}
#endif



/*
 * SetFlashFileSystemNumBlocks
 * This function defines the number of blocks to assign to the file system.
 * A minimum of 3 blocks is required because the first 2 are for storing
 * descriptors. FlashBlockSize() can tell you how many bytes are in each block.
 * FlashBlockSize() * numBlocks = the total space taken up by the file system.
 * Make sure there is enough space at the end of the flash after the program.
 * The file system is always placed at the back of the flash.
 *
 * If you change the number of blocks, you will likely have to format the file
 * system the first time you change the number of blocks.
 *
 * *DO NOT* call this function after the file system has been initialized.
 * It affects the starting address of the file system and thus will cause
 * memory corruption of any further file operations.
 */
char SetFlashFileSystemNumBlocks(int numBlocks)
{
	if(numBlocks > 2 && numBlocks < MAX_FS_BLOCKS)
	{
		NUM_FS_BLOCKS = numBlocks;
		return 1;
	}
	return 0;
}

/*
 * FlashBlockSize
 * This function returns the number of bytes in each flash block.
 * It can be used to determine how many blocks to allocate to the
 * file system.
 */
unsigned long FlashBlockSize()
{
	return BLOCKSIZE;
}


/*
 * MaxCodeSpaceAddrByBlock
 * This utility function tells you what the bounding address in memory will be
 * for a file system with the specified number of blocks.
 */
DWORD MaxCodeSpaceAddrByBlock(int numBlocks)
{
	return ( FLASH_SIZE - (numBlocks * BLOCKSIZE));
}


/*
 * MaxCodeSpaceAddr
 * This utility function tells you what the bounding address in memory will be
 * for a file system of size "fs_size" bytes.
 */
DWORD MaxCodeSpaceAddr(int fs_size)
{
	return ( FLASH_SIZE - (fs_size));
}

