/*
 * flashFS.h
 *
 *  Created on: Apr 29, 2014
 *      Author: Wesley
 */

#ifndef FLASHFS_H_
#define FLASHFS_H_

#include "file/fsf.h"
#include "basictypes.h"
#include "bsp.h"





/****************************************************************************
 *
 * fs_phy_OnChipFlash
 *
 * Identify a flash and fills FS_FLASH structure with data
 *
 * INPUTS
 *
 * flash - structure which is filled with data of flash properties
 *
 * RETURNS
 *
 * 0 - if successfully
 * 1 - flash not valid
 *
 ***************************************************************************/
int my_fs_phy_OnChipFlash( FS_FLASH *flash );


/*
 * OnChipFlash_GetFlashName
 * Returns the expected name of the flash chip in pName.
 */
void OnChipFlash_GetFlashName(char *pName );


/*
 * SetFlashFileSystemNumBlocks
 * This function defines the number of blocks to assign to the file system.
 * A minimum of 3 blocks is required because the first 2 are for storing
 * descriptors. FlashBlockSize() can tell you how many bytes are in each block.
 * FlashBlockSize() * numBlocks = the total space taken up by the file system.
 * Make sure there is enough space at the end of the flash after the program.
 * The file system is always placed at the back of the flash.
 *
 * If you change the number of blocks, you will likely have to format the file
 * system the first time you change the number of blocks.
 *
 * *DO NOT* call this function after the file system has been initialized.
 * It affects the starting address of the file system and thus will cause
 * memory corruption of any further file operations.
 */
char SetFlashFileSystemNumBlocks(int numBlocks);


/*
 * FlashBlockSize
 * This function returns the number of bytes in each flash block.
 * It can be used to determine how many blocks to allocate to the
 * file system.
 */
unsigned long FlashBlockSize();


/*
 * MaxCodeSpaceAddrByBlock
 * This utility function tells you what the bounding address in memory will be
 * for a file system with the specified number of blocks.
 */
DWORD MaxCodeSpaceAddrByBlock(int numBlocks);


/*
 * MaxCodeSpaceAddr
 * This utility function tells you what the bounding address in memory will be
 * for a file system of size "fs_size" bytes.
 */
DWORD MaxCodeSpaceAddr(int fs_size);






/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********  5234  ********  5272  ***********  SB72EX  **********  CB34EX  *******************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
#if ( defined MOD5234 || defined MOD5272 || defined SB72EX || defined CB34EX )
//#include "AM29LV160B.h"
////#include "AT49BV163D.h"   // Alternate to AM29LV160B
	#define FLASH_NAME   "S29AL016/AMDLV160"


	/**
	 * Start of flash memory base address.
	 */
	#define FS_FLASHBASE   ( 0xFFC00000 )


	/**
	 * BLOCKSIZE
	 * This defines the size of the blocks to be used in the file storage area.
	 * This must be an erasable unit of the flash chip.  All blocks in the file
	 * storage area must be the same size.  This may be different from the DESCSIZE
	 * where the flash chip has different size erasable units available.
	 *
	 * SECTORSIZE
	 * This defines the sector size.  Each block is divided into a number of
	 * sectors.  This number is the smallest usable unit in the system and thus
	 * represents the minimum file storage area.  For best usage of the flash
	 * blocks, the sector size should always be a power of two.  See sector section
	 * below for more information.
	 *
	 * SECTORPERBLOCK
	 * This defines the number of sectors in a block.  It must always be true that:
	 *
	 *    SECTORPERBLOCK = BLOCKSIZE / SECTORSIZE
	 *
	 * The memory map below is for a module with a 2MB bottom boot block flash.
	 * This example will allocate 1.5MB for the application space, and 512kB for the
	 * file system.  There are a total of 31 64kB blocks and 8 8kB blocks on the
	 * flash chip.
	 *
	 *                            Address
	 *                            -------
	 *    ----------------------  FFE0 0000 (End of flash space)
	 *    |  File System Data  |
	 *    |  384k              |
	 *    |  64k x 6 Blocks    |
	 *    |--------------------|  FFDA 0000 (Start of file system data)
	 *    |  DESC BLOCK 0/1    |
	 *    |  128k              |
	 *    |  64k x 2 Blocks    |
	 *    |--------------------|  FFD8 0000 (Start of file system)
	 *    |                    |
	 *    |  1.5M Application  |
	 *    |  64k x 23 Blocks   |
	 *    |                    |
	 *    |--------------------|
	 *    |  32k Application   |
	 *    |  8k x 4 Blocks     |
	 *    |--------------------|  FFC0 8000 (Start of application space)
	 *    |  8k User Params    |
	 *    |  8k x 1 Block      |
	 *    |--------------------|
	 *    |  8k System Params  |
	 *    |  8k x 1 Block      |
	 *    |--------------------|
	 *    |  16k Monitor       |
	 *    |  8k x 2 Blocks     |
	 *    |--------------------|  FFC0 0000 (Start of flash space)
	 */


	/**
	 * Warning:  These settings are for AM29LV160DB/S29AL016D bottom boot block
	 * flash components.
	 */
	#define BLOCKSIZE        (              64 * 1024 )  // Use only the 64k sectors
	#define SECTORSIZE       (              16 * 1024 )  // 4 sectors per block
	#define SECTORPERBLOCK   ( BLOCKSIZE / SECTORSIZE )


	/*
	 * Specify the total amount of flash memory in the system, and the amount
	 * allocated to be used by the file system (the rest is used by the
	 * application).
	 */
	#define FLASH_SIZE   (      2 * 1024 * 1024 )   // Size of total flash in the
													//    system, 2MB
	#define FS_SIZE    (           512 * 1024 )   // Amount allocated to file
													//    system, 512kB
	//#define FS_SIZE      (          1024 * 1024 )   // Amount allocated to file
													//    system, 1MB
	#define FIRST_ADDR   ( FLASH_SIZE - FS_SIZE )   // First file system address to
													//    use in the flash
	#define BLOCKSTART   (                    2 )   // First block where file system
													//    data starts (first 2
													//    blocks are DESCRIPTORS)


	/*
	 * Descriptor Blocks:
	 * These blocks contain critical information about the file system, block
	 * allocation, wear information, and file/directory information.  At least two
	 * descriptor blocks must be included in the system, which can be erased
	 * independently.  An optional descriptor write cache may be configured which
	 * improves the performance of the file system.  Please refer to the EFFS-STD
	 * implementation guide for additional information.
	 */
	#define DESCSIZE         ( 8 * 1024 )   // Size of one descriptor
	#define DESCBLOCKSTART   (        0 )   // Position of first descriptor
	#define DESCBLOCKEND     (        1 )   // Position of last descriptor
	#define DESCCACHE        (     2048 )


	#define PREALLOCATED_SPACE (32 * 1024)













/***********************************************************************************************************************/
/***********************************************************************************************************************/
/***********  5270B  ********  5270  ***********  SB70  **********  SB70LC  ********************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
#elif ( defined MOD5270B || defined MOD5270 || defined SB70 || defined SB70LC )
//#include "SST39VF040.h"

	#define FLASH_NAME   "SST39VF040"


	/**
	 * Start of flash memory base address.
	 */
	#define FS_FLASHBASE   ( 0xFFC00000 )


	/**
	 * BLOCKSIZE
	 * This defines the size of the blocks to be used in the file storage area.
	 * This must be an erasable unit of the flash chip.  All blocks in the file
	 * storage area must be the same size.  This may be different from the DESCSIZE
	 * where the flash chip has different size erasable units available.
	 *
	 * SECTORSIZE
	 * This defines the sector size.  Each block is divided into a number of
	 * sectors.  This number is the smallest usable unit in the system and thus
	 * represents the minimum file storage area.  For best usage of the flash
	 * blocks, the sector size should always be a power of two.  See sector section
	 * below for more information.
	 *
	 * SECTORPERBLOCK
	 * This defines the number of sectors in a block.  It must always be true that:
	 *
	 *    SECTORPERBLOCK = BLOCKSIZE / SECTORSIZE
	 *
	 * The memory map below is for a module with a 512kB bottom boot block flash.
	 * This example will allocate 448kB for the application and monitor space, and
	 * 64kB for the file system.  There are a total of 128 4kB blocks on the flash
	 * chip.
	 *
	 *                            Address
	 *                            -------
	 *    ----------------------  FFC8 0000 (End of flash space)
	 *    |  File System Data  |
	 *    |  56k               |
	 *    |  4k x 14 Blocks    |
	 *    |--------------------|  FFC7 2000 (Start of file system data)
	 *    |  DESC BLOCKS 0/1   |
	 *    |  8k                |
	 *    |  4k x 2 Blocks     |
	 *    |--------------------|  FFC7 0000 (Start of file system)
	 *    |                    |
	 *    |  Application       |
	 *    |  416k Bytes        |
	 *    |  4k x 104 Blocks   |
	 *    |                    |
	 *    |--------------------|  FFC0 8000 (Start of application space)
	 *    |  8k User Params    |
	 *    |  4k x 2 Blocks     |
	 *    |--------------------|
	 *    |  8k System Params  |
	 *    |  4k x 2 Blocks     |
	 *    |--------------------|
	 *    |  16k Monitor       |
	 *    |  4k x 4 Blocks     |
	 *    |--------------------|  FFC0 0000 (Start of flash space)
	 */

	/**
	 * Warning:  These settings are for SST 39VF040 bottom boot block flash
	 * components.
	 */
	#define BLOCKSIZE        (               4 * 1024 )  // Use only the 64k sectors
	#define SECTORSIZE       (                    512 )  // 8 sectors per block
	#define SECTORPERBLOCK   ( BLOCKSIZE / SECTORSIZE )


	/*
	 * Specify the total amount of flash memory in the system, and the amount
	 * allocated to be used by the file system (the rest is used by the
	 * application).
	 */
	#define FLASH_SIZE   (           512 * 1024 )   // Size of total flash in the
													//    system, 512kB
	#define FS_SIZE      (            64 * 1024 )   // Amount allocated to file
													//    system, 64kB
	#define FIRST_ADDR   ( FLASH_SIZE - FS_SIZE )   // First file system address to
													//    use in the flash
	#define BLOCKSTART   (                    2 )   // First block where file system
													//    data starts (first 2
													//    blocks are DESCRIPTORS)


	/*
	 * Descriptor Blocks:
	 * These blocks contain critical information about the file system, block
	 * allocation, wear information, and file/directory information.  At least two
	 * descriptor blocks must be included in the system, which can be erased
	 * independently.  An optional descriptor write cache may be configured which
	 * improves the performance of the file system.  Please refer to the EFFS-STD
	 * implementation guide for additional information.
	 */
	#define DESCSIZE         ( 4 * 1024 )   // Size of one descriptor
	#define DESCBLOCKSTART   (        0 )   // Position of first descriptor
	#define DESCBLOCKEND     (        1 )   // Position of last descriptor
	#define DESCCACHE        (     1024 )


	#define PREALLOCATED_SPACE (32 * 1024)












/***********************************************************************************************************************/
/***********************************************************************************************************************/
/**********************************************  PK70  *****************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
#elif ( defined PK70 )
//#include "S29GL032.h"

	#define FLASH_NAME   "S29GL032A"

	/* Start of flash memory base address */
	#define FS_FLASHBASE   ( 0xFF800000 )


	/**
	 * BLOCKSIZE
	 * This defines the size of the blocks to be used in the file storage area.
	 * This must be an erasable unit of the flash chip.  All blocks in the file
	 * storage area must be the same size.  This may be different from the DESCSIZE
	 * where the flash chip has different size erasable units available.
	 *
	 * SECTORSIZE
	 * This defines the sector size.  Each block is divided into a number of
	 * sectors.  This number is the smallest usable unit in the system and thus
	 * represents the minimum file storage area.  For best usage of the flash
	 * blocks, the sector size should always be a power of two.  See sector section
	 * below for more information.
	 *
	 * SECTORPERBLOCK
	 * This defines the number of sectors in a block.  It must always be true that:
	 *
	 *    SECTORPERBLOCK = BLOCKSIZE / SECTORSIZE
	 *
	 * The memory map below is for a module with a 4MB bottom boot block flash.
	 * This example will allocate 3MB for the application space, and 1MB for the
	 * file system.  There are a total of 63 64kB blocks and 8 8kB blocks on the
	 * flash chip.
	 *
	 *                            Address
	 *                            -------
	 *    ----------------------  FFC0 0000 (End of flash space)
	 *    |  File System Data  |
	 *    |  917,504 Bytes     |
	 *    |  64k x 14 Blocks   |
	 *    |--------------------|  FFB2 0000 (Start of file system data)
	 *    |  DESC BLOCK 0/1    |
	 *    |  131,072 Bytes     |
	 *    |  64k x 2 Blocks    |
	 *    |--------------------|  FFB0 0000 (Start of file system)
	 *    |                    |
	 *    |  2.81M Application |
	 *    |  64k x 45 Blocks   |
	 *    |                    |
	 *    |--------------------|  FF83 0000 (Start of application space)
	 *    |  64k User Params   |
	 *    |  64k x 1 Block     |
	 *    |--------------------|
	 *    |  64k System Params |
	 *    |  64k x 1 Block     |
	 *    |--------------------|
	 *    |  64k Monitor       |
	 *    |  8k x 8 Blocks     |
	 *    |--------------------|  FF80 0000 (Start of flash space)
	 */


	/**
	 * Warning:  These settings are for AM29LV160DB/S29AL016D bottom boot block
	 * flash components.
	 */
	#define BLOCKSIZE        (              64 * 1024 )  // Use only the 64k sectors
	#define SECTORSIZE       (              16 * 1024 )  // 4 sectors per block
	#define SECTORPERBLOCK   ( BLOCKSIZE / SECTORSIZE )


	/*
	 * Specify the total amount of flash memory in the system, and the amount
	 * allocated to be used by the file system (the rest is used by the
	 * application).
	 */
	#define FLASH_SIZE   (      4 * 1024 * 1024 )   // Size of total flash in the system, 4MB
	#define FS_SIZE      (          1024 * 1024 )   // Amount allocated to file system, 1MB
	#define FIRST_ADDR   ( FLASH_SIZE - FS_SIZE )   // First file system address to use in the flash
	#define BLOCKSTART   ( 2 )   // First block where file system data starts (first 2 blocks are DESCRIPTORS)


	/*
	 * Descriptor Blocks:
	 * These blocks contain critical information about the file system, block
	 * allocation, wear information, and file/directory information.  At least two
	 * descriptor blocks must be included in the system, which can be erased
	 * independently.  An optional descriptor write cache may be configured which
	 * improves the performance of the file system.  Please refer to the EFFS-STD
	 * implementation guide for additional information.
	 */
	#define DESCSIZE         ( 8 * 1024 )   // Size of one descriptor
	#define DESCBLOCKSTART   (        0 )   // Position of first descriptor
	#define DESCBLOCKEND     (        1 )   // Position of last descriptor
	#define DESCCACHE        (     2048 )


	#define PREALLOCATED_SPACE ((64*3) * 1024)











/***********************************************************************************************************************/
/***********************************************************************************************************************/
/**********************************************  5282  *****************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
#elif ( defined MOD5282 )
//#include "MCF5282Flash.h"

	#define FLASH_NAME   "MCF5282"      // Processor name, not module name


	/**
	 * Start of flash memory base address.
	 */
	#define FS_FLASHBASE   ( 0xFFC00000 )


	/**
	 * BLOCKSIZE
	 * This defines the size of the blocks to be used in the file storage area.
	 * This must be an erasable unit of the flash chip.  All blocks in the file
	 * storage area must be the same size.  This may be different from the DESCSIZE
	 * where the flash chip has different size erasable units available.
	 *
	 * SECTORSIZE
	 * This defines the sector size.  Each block is divided into a number of
	 * sectors.  This number is the smallest usable unit in the system and thus
	 * represents the minimum file storage area.  For best usage of the flash
	 * blocks, the sector size should always be a power of two.  See sector section
	 * below for more information.
	 *
	 * SECTORPERBLOCK
	 * This defines the number of sectors in a block.  It must always be true that:
	 *
	 *    SECTORPERBLOCK = BLOCKSIZE / SECTORSIZE
	 *
	 * The memory map below is for a module with a 512kB bottom boot block flash.
	 * This example will allocate 448kB for the application and monitor space, and
	 * 64kB for the file system.  There are a total of 128 4kB blocks on the flash
	 * chip.
	 *
	 *                            Address
	 *                            -------
	 *    ----------------------  FFC8 0000 (End of flash space)
	 *    |  File System Data  |
	 *    |  56k               |
	 *    |  4k x 14 Blocks    |
	 *    |--------------------|  FFC7 2000 (Start of file system data)
	 *    |  DESC BLOCKS 0/1   |
	 *    |  8k                |
	 *    |  4k x 2 Blocks     |
	 *    |--------------------|  FFC7 0000 (Start of file system)
	 *    |                    |
	 *    |  Application       |
	 *    |  416k Bytes        |
	 *    |  4k x 104 Blocks   |
	 *    |                    |
	 *    |--------------------|  FFC0 8000 (Start of application space)
	 *    |  8k User Params    |
	 *    |  4k x 2 Blocks     |
	 *    |--------------------|
	 *    |  8k System Params  |
	 *    |  4k x 2 Blocks     |
	 *    |--------------------|
	 *    |  16k Monitor       |
	 *    |  4k x 4 Blocks     |
	 *    |--------------------|  FFC0 0000 (Start of flash space)
	 */

	/**
	 * Warning:  These settings are for a MCF5282 bottom boot block flash
	 */
	#define BLOCKSIZE        (               4 * 1024 )  // All sectors are 4KB
	#define SECTORSIZE       (                    512 )  // 8 sectors per block
	#define SECTORPERBLOCK   ( BLOCKSIZE / SECTORSIZE )


	/*
	 * Specify the total amount of flash memory in the system, and the amount
	 * allocated to be used by the file system (the rest is used by the
	 * application).
	 */
	#define FLASH_SIZE   (           512 * 1024 )   // Size of total flash in the
													//    system, 512kB
	#define FS_SIZE      (            64 * 1024 )   // Amount allocated to file
													//    system, 64kB
	#define FIRST_ADDR   ( FLASH_SIZE - FS_SIZE )   // First file system address to
													//    use in the flash
	#define BLOCKSTART   (                    2 )   // First block where file system
													//    data starts (first 2
													//    blocks are DESCRIPTORS)


	/*
	 * Descriptor Blocks:
	 * These blocks contain critical information about the file system, block
	 * allocation, wear information, and file/directory information.  At least two
	 * descriptor blocks must be included in the system, which can be erased
	 * independently.  An optional descriptor write cache may be configured which
	 * improves the performance of the file system.  Please refer to the EFFS-STD
	 * implementation guide for additional information.
	 */
	#define DESCSIZE         ( 4 * 1024 )   // Size of one descriptor
	#define DESCBLOCKSTART   (        0 )   // Position of first descriptor
	#define DESCBLOCKEND     (        1 )   // Position of last descriptor
	#define DESCCACHE        (     1024 )

	#define PREALLOCATED_SPACE (32 * 1024)



/***********************************************************************************************************************/
/***********************************************************************************************************************/
/********************************************  NANO54415  **************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
#elif defined NANO54415
#define FLASH_NAME   "MX25L6406E"


/**
 * BLOCKSIZE
 * This defines the size of the blocks to be used in the file storage area.
 * This must be an erasable unit of the flash chip.  All blocks in the file
 * storage area must be the same size.  This may be different from the DESCSIZE
 * where the flash chip has different size erasable units available.
 *
 * SECTORSIZE
 * This defines the sector size.  Each block is divided into a number of
 * sectors.  This number is the smallest usable unit in the system and thus
 * represents the minimum file storage area.  For best usage of the flash
 * blocks, the sector size should always be a power of two.  See sector section
 * below for more information.
 *
 * SECTORPERBLOCK
 * This defines the number of sectors in a block.  It must always be true that:
 *
 *    SECTORPERBLOCK = BLOCKSIZE / SECTORSIZE
 *
 * The memory map below is for a module with a 512kB bottom boot block flash.
 * This example will allocate 448kB for the application and monitor space, and
 * 64kB for the file system.  There are a total of 128 4kB blocks on the flash
 * chip.
 *
 *                            Address In SPI Flash
 *                            -------
 *    ----------------------  0080 0000  (End of flash space)
 *    |  File System Data  |
 *    |  56k               |
 *    |  4k x 14 Blocks    |
 *    |--------------------|  007F 2000 (Start of file system data)
 *    |  DESC BLOCKS 0/1   |
 *    |  8k                |
 *    |  4k x 2 Blocks     |
 *    |--------------------|  007F 0000 (Start of file system)
 *    |                    |
 *    |  Application       |
 *    |   Bytes            |
 *    |  4k x 2028 Blocks  |
 *    |                    |
 *    |--------------------|  4000
 *    |  8k User Params  |
 *    |  4k x 2 Blocks     |
 *    |--------------------|  2000
 *    |  8k Global config  |
 *    |  4k x 4 Blocks     |
 *    |--------------------|  0000 (Start of flash space)
 */

 /*
 * CHANGES TO COMPCODE FLAGS
 * In NBEclipse, or your command line makefile, change the following line
 * so the application will only occupy the specified application space.
 * The first parameter is the start of application space, and the second
 * is the address just below the file system space.
 *
 *    COMPCODEFLAGS = 0x04000 0x800000		// Original
 *
 *    COMPCODEFLAGS = 0x04000 0x7F0000		// space for file system
*/


#define BLOCKSIZE        (               4 * 1024 )  // All sectors are 4KB
#define SECTORSIZE       (                    512 )  // 8 sectors per block
#define SECTORPERBLOCK   ( BLOCKSIZE / SECTORSIZE )


/*
 * Specify the total amount of flash memory in the system, and the amount
 * allocated to be used by the file system (the rest is used by the
 * application).
 */
#define FLASH_SIZE   (        8*1024 * 1024 )   // Total flash space
//#define FS_SIZE      (            64 * 1024 )   // Amount of Flash allocated to file system
//#define FS_SIZE      (            560 * 1024 )   // Amount of Flash allocated to file system
#define FS_SIZE      (        1024 * 1024*3 )   // Amount of Flash allocated to file system
#define FIRST_ADDR   ( FLASH_SIZE - FS_SIZE )   // First Flash file system address
//#define BLOCKSTART   (                    2 )   // First block where file system data starts (first 2 blocks are DESCRIPTORS)
#define BLOCKSTART   (                    16 )   // First block where file system data starts (first 2 blocks are DESCRIPTORS)


/*
 * Descriptor Blocks:
 * These blocks contain critical information about the file system, block
 * allocation, wear information, and file/directory information.  At least two
 * descriptor blocks must be included in the system, which can be erased
 * independently.  An optional descriptor write cache may be configured which
 * improves the performance of the file system.  Please refer to the EFFS-STD
 * implementation guide for additional information.
 */
//#define DESCSIZE         ( 4 * 1024 )   // Size of one descriptor
#define DESCSIZE         ( 32 * 1024 )   // Size of one descriptor
#define DESCBLOCKSTART   (        0 )   // Position of first descriptor
#define DESCBLOCKEND     (        1 )   // Position of last descriptor
//#define DESCBLOCKEND     (        3 )   // Position of last descriptor
//#define DESCCACHE        (     1024 )
#define DESCCACHE        (  1024 )


#define PREALLOCATED_SPACE (16 * 1024)






/***********************************************************************************************************************/
/***********************************************************************************************************************/
/********************************************  MOD5441X  ***************************************************************/
/***********************************************************************************************************************/
/***********************************************************************************************************************/
#elif defined MOD5441X

#define FLASH_NAME   "MX29GL256F"

// Start of Flash memory base address
#define FS_FLASHBASE (0xC0000000)

/*
 * BLOCKSIZE
 * This defines the size of the blocks to be used in the file storage area.
 * This must be an erasable unit of the flash chip. All blocks in the file
 * storage area must be the same size. This maybe different from the DESCSIZE
 * where the flash chip has different size erasable units available.
 *
 * SECTORSIZE
 * This defines the sector size. Each block is divided into a number of sectors.
 * This number is the smallest usable unit in the system and thus represents the
 * minimum file storage area. For best usage of the flash blocks the sector size
 * should always be a power of 2. For more information see sector section below.
 *
 * SECTORPERBLOCK
 * This defines the number of sectors in a block. It must always be true that:
 * SECTORPERBLOCK = BLOCKSIZE/SECTORSIZE
 *
 *
 * The memory map below is for a Mod5234 with a 2MB bottom boot block flash.
 * This example will allocate 1.5MB for the application space, and 512KB for
 * the file system. There are a total of 31 64K blocks on the flash chip.
 *
 * Macronix MX29GL256F, 256 blocks of 128KB each, Total of 32MBytes
 *
 *                             Address
 *                            ----------
 *    ---------------------- C1FFF FFFF (End of flash space)
 *    |  File System Data  |
 *    |  1MB               |
 *    |  128K x 8 Blocks   |
 *    |--------------------| C1F0 0000 (Start of File System Data)
 *    |  DESC BLOCK 0/1    |
 *    | 128K x 2 Blocks    |
 *    |--------------------| C1EC 0000 (Start of File System)
 *    |                    |
 *    | Application        |
 *    | 31.232MB           |
 *    | 128K x 244 Blocks  |
 *    |                    |
 *    |--------------------| C004 0000
 *    | 128K User Params   |
 *    |--------------------| C002 0000
 *    | 128K System Params |
 *    |--------------------| C000 0000 (Start of Flash space)
 *
 *
 * CHANGES TO COMPCODE FLAGS
 * In NBEclipse, or your command line makefile, change the following line
 * so the application will only occupy the specified application space.
 * The first parameter is the start of application space, and the second
 * is the address just below the file system space.
 *
 *    COMPCODEFLAGS = 0xC0040000 0xC1EC0000
 *
 * If using NBEclipse:
 * - Right-click on the project and select "Properties"
 * - Select "NetBurner" in the left side of the dialog box
 * - Verify the Platform is set to Mod5234, then check the "Use Custom Platform Settings" checkbox
 * - Modify the "Compcode Memory Range" to the above values
 *
 *
 * If using NBEclipse, you will also need to tell the linker to include
 * the c:\nburn\lib\StdFFile.a library. To do this right-click on your
 * project, select properties, GNU Linker, then add the library.
 *
 */

/* WARNING: These settings are for MX29GL256F bottom boot block flash
 * components used on the Mod54415
 */
#define BLOCKSIZE  ( 128*1024 )   // flash physical "sector" size
#define SECTORSIZE ( 1*1024 )     // file system sectors per BLOCK
#define SECTORPERBLOCK (BLOCKSIZE/SECTORSIZE)

/*
 * Specify the total amount of flash memory in the system, and the amount
 * allocated to be used by the file system (the rest is used by the
 * application.
 */
#define FLASH_SIZE ( 32*1024*1024 )       // size of total flash in the system, 32MB
#define FS_SIZE    ( 1280*1024 )          // amount allocated to file system: 2 Desc. plus 1MB for data (8 x 128k)
                                          // note that 1 block of file data will be reserved for the file system
#define FIRST_ADDR (FLASH_SIZE - FS_SIZE) // first file system address to use in the flash
#define BLOCKSTART 2                      // first block where file system data starts
                                          // (first 2 blocks are DESCRIPTORS)


/*
 * Descriptor Blocks:
 * These blocks contain critical information about the file system, block allocation,
 * wear information and file/directory information. At least two descriptor blocks
 * must be included in the system, which can be erased independently. An optional
 * descriptor write cache may be configured which improves the performance of the
 * file system. Please refer to the EFFS-STD implementation guide for additional
 * information.
 */
#define DESCSIZE ( 64*1024 )        // size of one descriptor
#define DESCBLOCKSTART 0           // position of first descriptor
#define DESCBLOCKEND 1             // position of last descriptor
#define DESCCACHE 2048

#define PREALLOCATED_SPACE (256 * 1024)


#else
#error "*** EFFS STD FLASH PLATFORM NOT DEFINED ***"
#endif


#endif /* FLASHFS_H_ */
